<?php

namespace App\Form\Type;

use App\Entity\Flux;
use App\Entity\User;
use App\Enum\MoyenEnum;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\MoneyType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Security;
use Symfony\Component\Validator\Constraints\LessThanOrEqual;

class FluxFormType extends AbstractType
{
    protected $em;
    protected $security;

    public function __construct(EntityManagerInterface $em, Security $security)
    {
        $this->em = $em;
        $this->security = $security;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $flux = $options['data'];
        if (empty($this->security->getUser()) || empty($this->security->getUser()->getId()) || empty($flux) || empty($flux->getExpediteur())) {
            throw new \Exception("[FLUX] Opération impossible ! Pas d'utilisateur connecté !");
        }
        $maxvalue = $flux->getExpediteur()->getCompte();
        $builder
            ->add('operateur', HiddenType::class, array(
                'data' => $this->security->getUser()->getId(),
                'data_class' => null,
                'entity_class' => User::class,
                'em' => $this->em
            ))
            ->add('montant', MoneyType::class, array(
                'label' => 'Montant :',
                'required' => true,
                'constraints' => [
                    new LessThanOrEqual([
                        'message' => 'Montant supérieur à votre solde.',
                        'value' => $maxvalue,
                    ]),
                ],
            ))
            ->add('reference', TextType::class, array(
                'label' => 'Reference :',
                'required' => true
            ))
            ->add('save', SubmitType::class, ['label' => "Valider"])
            ->addEventListener(FormEvents::POST_SUBMIT, function (FormEvent $event) use ($builder) {
                $flux = $event->getData();
                if ($event->getForm()->isValid() && (!$flux || null === $flux->getId())) {
                    if ($flux->getExpediteur()->getCompte() < $flux->getMontant()) {
                        $event->getForm()->get('montant')->addError(new FormError("Montant supérieur à votre solde !"));
                    }
                }
            })
        ;
    }

    /**
     * {@inheritdoc}
     */
    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(array(
            'data_class' => Flux::class,
            'cascade_validation' => true
        ));
    }

    public function getBlockPrefix()
    {
        return 'formFlux';
    }
}
